import pygame

# Constants
C_GRID_L_VALUE = 200
C_GRID_D_VALUE = 100
C_MPLOT_START = 700
C_BLINK_TIME = 500

gridLight = pygame.Color(C_GRID_L_VALUE, C_GRID_L_VALUE, C_GRID_L_VALUE)
gridDark = pygame.Color(C_GRID_D_VALUE, C_GRID_D_VALUE, C_GRID_D_VALUE)
font_h = pygame.font.SysFont(None, 28)
font_m = pygame.font.SysFont(None, 16)

# UI Class
class SimUI:
    def __init__(self, screen, pole):
        self.screen = screen
        self.pole = pole

        self.metaPlotY = C_MPLOT_START
        
        self.blink = False
        self.blinkTimer = 0
        
        self.controlled = True
        self.paused = False

    def meta(self, val, desc):
        # Capture values to display status
        if desc == "Control":
            self.controlled = val
        if desc == "Paused":
            self.paused = val
        
        # Print em
        self.screen.blit(
            font_m.render(f"{desc} = {val}", True, "black"), (10, self.metaPlotY)
        )
        self.metaPlotY -= 15

    def grid(self, dist, Xoff=0, Yoff=0):
        # Clear the screen
        self.screen.fill("white")
        
        # Drawing offsets so the grid aligns with the pole
        cXoff = self.pole.x % dist
        cYoff = self.pole.y % dist

        # Draw the grid
        for i in range(0, 1280, dist):
            pygame.draw.line(
                self.screen,
                gridLight,
                (i + Xoff + cXoff, 0),
                (i + Xoff + cXoff, 720),
                1,
            )
            pygame.draw.line(
                self.screen,
                gridLight,
                (0, i + Yoff + cYoff),
                (1280, i + Yoff + cYoff),
                1,
            )

        # Draw the center lines darker
        pygame.draw.line(
            self.screen, gridDark, (self.pole.x + Xoff, 0), (self.pole.x + Xoff, 720), 1
        )
        pygame.draw.line(
            self.screen,
            gridDark,
            (0, self.pole.y + Yoff),
            (1280, self.pole.y + Yoff),
            1,
        )

    def centeredText(self, font, text="", colour="black", y=0):
        textObj = font.render(text, True, colour)
        text_rect = textObj.get_rect(center=(1280 / 2, 720 / 2 - y))
        self.screen.blit(textObj, text_rect)

    def gameover(self, time):
        font_g = pygame.font.Font("res\\pricedown.otf", 128)
        self.centeredText(font_g, "wasted", "red", 150)
        self.centeredText(font_m, f"You controlled the pendulum for {time / 1000} seconds", "black", 80)
        
        self.centeredText(font_m, "Press space to restart", "black", 60)
        self.centeredText(font_m, "Press G to view nerd graphs", "black", 45)

    def update(self, dt):
        # Credits
        self.screen.blit(
            font_h.render("Pendulum simulator 4000", True, "black"), (10, 10)
        )
        self.screen.blit(
            font_m.render("Arne van Iterson, 2023", True, "black"), (1150, 700)
        )
        
        # Reset meta printing
        self.metaPlotY = C_MPLOT_START
        
        # Draw current status blinking
        if self.blink:
            if self.paused:
                text = "Paused"
            elif self.controlled:
                text = "Auto mode"
            else:
                text = "Manual mode"
                
            textObj = font_h.render(text, True, "black")
            text_rect = textObj.get_rect(right=1270, top=10)
            self.screen.blit(textObj, text_rect)

        if self.blinkTimer > C_BLINK_TIME:
            self.blink = not self.blink
            self.blinkTimer = 0
        self.blinkTimer += dt
